@@ -46,10 +46,6 @@ def check_dataframe_deps():
         warnings.warn(pa_error)
 
 
-# TODO
-# - handle missing values
-# - handle extended datatypes
-
 # Note: 'None' is used to indicate optionality for many of these options
 #       For example, if the `sparse` argument is unspecified we will default
 #       to False (dense) unless the input has string or heterogenous indexes.
@@ -86,6 +82,24 @@ def parse_tiledb_kwargs(kwargs):
     return parsed_args
 
 
+def is_nullable_extension_type(t):
+    import pandas as pd
+
+    return isinstance(
+        t,
+        (
+            pd.Int64Dtype,
+            pd.Int32Dtype,
+            pd.Int16Dtype,
+            pd.Int8Dtype,
+            pd.UInt64Dtype,
+            pd.UInt32Dtype,
+            pd.UInt16Dtype,
+            pd.UInt8Dtype,
+        ),
+    )
+
+
 class ColumnInfo:
     def __init__(self, dtype, repr: Optional[str] = None, nullable: bool = False):
         self.dtype = dtype
@@ -108,19 +122,7 @@ def dtype_from_column(col):
     ):
         return ColumnInfo(col_dtype)
 
-    if isinstance(
-        col_dtype,
-        (
-            pd.Int64Dtype,
-            pd.Int32Dtype,
-            pd.Int16Dtype,
-            pd.Int8Dtype,
-            pd.UInt64Dtype,
-            pd.UInt32Dtype,
-            pd.UInt16Dtype,
-            pd.UInt8Dtype,
-        ),
-    ):
+    if is_nullable_extension_type(col_dtype):
         return ColumnInfo(col_dtype.numpy_dtype, repr=str(col_dtype), nullable=True)
 
     if isinstance(col_dtype, pd.BooleanDtype):
@@ -191,13 +193,15 @@ def attrs_from_df(df, index_dims=None, filters=None, column_types=None, ctx=None
         # ignore any column used as a dim/index
         if index_dims and name in index_dims:
             continue
-
         if name in column_types:
             spec_type = column_types[name]
+            nullable = is_nullable_extension_type(spec_type)
+
+            actual_type = spec_type
             # Handle ExtensionDtype
             if hasattr(spec_type, "type"):
-                spec_type = spec_type.type
-            attr_info = ColumnInfo(spec_type)
+                actual_type = spec_type.type
+            attr_info = ColumnInfo(actual_type, repr=str(spec_type), nullable=nullable)
         else:
             attr_info = dtype_from_column(col)
 
@@ -359,8 +363,8 @@ def create_dims(
             and not all(map(lambda x: isinstance(x, (int, float)), tile.values()))
         ) or (per_dim_tile is False and not isinstance(tile, (int, float))):
             raise ValueError(
-                "Invalid tile kwarg: expected int or tuple of ints "
-                "got '{}'".format(tile)
+                "Invalid tile kwarg: expected int or dict of column names mapped to ints. "
+                "Got '{}'".format(tile)
             )
 
     if isinstance(index, pd.MultiIndex):
@@ -510,28 +514,27 @@ def write_array_metadata(array, attr_metadata=None, index_metadata=None):
 def dataframe_to_np_arrays(dataframe, fillna=None):
     import pandas as pd
 
-    if hasattr(pd, "StringDtype"):
-        # version > 1.0. StringDtype introduced in pandas 1.0
-        ret = dict()
-        nullmaps = dict()
-        for k, v in dataframe.to_dict(orient="series").items():
-            if pd.api.types.is_extension_array_dtype(v):
-                #
-                if fillna is not None and k in fillna:
-                    # raise ValueError("Missing 'fillna' value for column '{}' with pandas extension dtype".format(k))
-                    ret[k] = v.to_numpy(na_value=fillna[k])
-                else:
-                    # use default 0/empty for the dtype
-                    ret[k] = v.to_numpy(
-                        dtype=v.dtype.numpy_dtype, na_value=v.dtype.type()
-                    )
-                    nullmaps[k] = (~v.isna()).to_numpy(dtype="uint8")
+    if not hasattr(pd, "StringDtype"):
+        raise Exception(
+            "Unexpectedly found pandas version < 1.0; please install >= 1.0 for dataframe functionality."
+        )
+
+    ret = dict()
+    nullmaps = dict()
+
+    for k, v in dataframe.to_dict(orient="series").items():
+        if pd.api.types.is_extension_array_dtype(v):
+            if (fillna is not None and k in fillna) and not is_nullable_extension_type(
+                v.dtype
+            ):
+                # raise ValueError("Missing 'fillna' value for column '{}' with pandas extension dtype".format(k))
+                ret[k] = v.to_numpy(na_value=fillna[k])
             else:
-                ret[k] = v.to_numpy()
-    else:
-        # version < 1.0
-        nullmaps = dict()
-        ret = {k: v.values for k, v in dataframe.to_dict(orient="series").items()}
+                # use default 0/empty for the dtype
+                ret[k] = v.to_numpy(dtype=v.dtype.numpy_dtype, na_value=v.dtype.type())
+                nullmaps[k] = (~v.isna()).to_numpy(dtype="uint8")
+        else:
+            ret[k] = v.to_numpy()
 
     return ret, nullmaps
 
@@ -547,13 +550,17 @@ def from_dataframe(uri, dataframe, **kwargs):
 
 
 def from_pandas(uri, dataframe, **kwargs):
-    """Create TileDB array at given URI from pandas dataframe
+    """Create TileDB array at given URI from a Pandas dataframe
+
+    Supports most Pandas series types, including nullable integers and
+    bools.
 
     :param uri: URI for new TileDB array
     :param dataframe: pandas DataFrame
     :param mode: Creation mode, one of 'ingest' (default), 'schema_only', 'append'
 
-    :Keyword Arguments: optional keyword arguments for TileDB, see ``tiledb.from_csv``.
+    :Keyword Arguments: optional keyword arguments for TileDB conversion, see
+    ``tiledb.from_csv`` for additional details.
 
     :raises: :py:exc:`tiledb.TileDBError`
     :return: None
@@ -851,8 +858,8 @@ def from_csv(uri, csv_file, **kwargs):
                 for any dimensions(s). Unspecified dimensions will use default.
             * ``coords_filters``: FilterList to apply to all coordinates (Dimensions)
             * ``sparse``: (default True) Create sparse schema
-            * ``tile``: Dimension tiling: accepts either Int or a list of Tuple[Int] with per-dimension
-              'tile' arguments to apply to the generated ArraySchema.
+            * ``tile``: Dimension tiling: accepts either an int that applies the tiling to all dimensions
+                or a dict("dim_name": int) to specifically assign tiling to a given dimension
             * ``capacity``: Schema capacity.
             * ``timestamp``: Write TileDB array at specific timestamp.
             * ``row_start_idx``: Start index to start new write (for row-indexed ingestions).
